#ifndef _mesh_3ds_
#define _mesh_3ds_

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <string.h>
#include "vector2.h"

#define MAIN3DS					0x4D4D
#define KEYF3DS					0xB000  // this is the start of the keyframer config
#define	VERSION					0x0002
#define MESH_VERSION			0x3D3E


#define	EDIT3DS					0x3D3D
#define	NAMED_OBJECT			0x4000	// level 2
#define	TRIANGLE_MESH			0x4100	// level 3
#define OBJ_LIGHT				0x4600	// level 3
#define OBJ_CAMERA				0x4700	// level 3

#define TRIANGLE_VERTEXLIST		0x4110	// level 4
#define	TRIANGLE_FACELIST		0x4120	// level 4
#define	TRIANGLE_MATERIAL		0x4130  // level 5
#define TRI_SMOOTH				0x4150	// level 5
#define	TRIANGLE_MAPPINGCOORS	0x4140	// level 4 


#define EDIT_MATERIAL			0xAFFF	// level 2
#define MATNAME					0xA000	// This holds the material name
#define MATERIAL_AMBIENT		0xA010	// This holds the material name
#define MATERIAL_DIFFUSE		0xA020	// This holds the color of the object/material
#define	MATERIAL_SPECULAR		0xA030
#define	MATERIAL_SHININESS		0xA040
#define	MATERIAL_SHIN_STRENGTH	0xA041
#define	TRANS_PERCENT			0xA050
#define	TRANS_FALLOFF_PERCENT	0xA052
#define	REFLECTION_BLUR_PER		0xA053
#define	SELF_ILLUM				0xA084
#define	RENDER_TYPE				0xA100  // level 4?
#define	WIRE_THICKNESS			0xA087
#define	TEXTURE_MAP				0xA200  // level 4?
#define	TEXTURE_MAP1			0xA204  // level 4?
#define	MAP_SPECULAR			0xA230  // level 4?
#define	MAPPING_NAME			0xA300  // level 5?
#define	MAPPING_1				0x0030  // level 5?
#define EDIT_AMBIENT			0x2100	// level 2

struct chunk
{
	unsigned short label;			//   - 2 
	unsigned int length;			//   - 4 
	unsigned int bytes;				//       
	unsigned int last_ptr;
	unsigned int next_ptr;
};


struct Face
{
	Vector3D v0;
	Vector3D v1;
	Vector3D v2;
	Vector2D st0;
	Vector2D st1;
	Vector2D st2;
	Vector3D Norm;
	Vector3D Binorm;
	Vector3D Tangent;
};


struct tFace_PPL
{

	Vector2D st0;
	Vector2D st1;
	Vector2D st2;
	Vector3D n0;
	Vector3D n1;
	Vector3D n2;
	Vector3D v0;
	Vector3D v1;
	Vector3D v2;
	Vector3D b0;
	Vector3D b1;
	Vector3D b2;
	Vector3D t0;
	Vector3D t1;
	Vector3D t2;
};

struct tFace
{
	Vector2D st0;
	Vector3D n0;
	Vector3D v0;
	Vector2D st1;
	Vector3D n1;
	Vector3D v1;
	Vector2D st2;
	Vector3D n2;
	Vector3D v2;
};


struct Vertex
{
	Vector3D v;
	Vector2D st;
	Vector3D Norm;
	Vector3D Binorm;
	Vector3D Tangent;
};

struct T3DObjectMesh
{
	int				ObjectID;
	int				TextureID;
	Vector3D		min;
	Vector3D		max;
	int				num_vertex;
	int				num_vertex_st;
	int				num_face;
	char		*	strObjectName;
	char		*	strTextureName;
	int			*	pIndFaceArray;
	int			*	pSmoothFaceArray;
	Vector3D	*	pVertex;
	Vector2D	*	pVertexST;
	Vector3D	*	pVertexArray;
	Vector3D	*	pTangentArray;
	Vector3D	*	pBinormalArray;
	Vector3D	*	pNormalArray;
	Vector2D	*	pVertexSTArray;
	tFace		*	pFace;
};


struct T3DMaterialMesh
{
	char  * strMaterialName;
	char  * strFileName;
	int   MaterialId;
};

struct T3DScene
{
	int num_objects;
	int num_materials;
	T3DObjectMesh *pObject;
	T3DMaterialMesh *pMaterial;
};


class mesh3ds
{
private:
		int ObjID;
		int MatID;
		int num_obj;
		int num_mtr;
		FILE *P_File;
		FILE *P_LogFile;	
public:
		T3DScene * pScene;

public:
		mesh3ds();
		~mesh3ds();
		char ReadChar(void);
		unsigned int ReadChunk(chunk *p_chunk);
		void SetChunkPointer(unsigned int ptr);
		unsigned long GetChunkPosition();
		void SetChunkPosition(unsigned int ptr);
		void Get3DSFileInfo(chunk *p_chunk);
		void ReadMapChunk(chunk *p_chunk);
		void ReadMapSpecular(chunk *p_chunk);
		void ReadEditMaterialChunk(chunk *p_chunk);
		unsigned int ReadObjectName (void);
		unsigned int ReadTextureName (void);
		unsigned int ReadMaterialName (void);
		unsigned int ReadFileName (void);
		void ReadVertexData(void);
		void ReadMappingData(void);
		void ReadFaceData(chunk *p_chunk);
		void ReadTriangleMeshChunk(chunk *p_chunk);
		void ReadNamedObjectChunk(chunk *p_chunk);
		void ReadEdit3DSChunk(chunk *p_chunk);
		void MakeTangentSpace(void);
		void MakeFaceNormals(void);
		void CompareMaterials(void);
		T3DScene * Load3ds(const char *name);

};





#endif _mesh_3ds_