#include "opengl.h"

COpenGL::COpenGL()
{
	HDC	hDC = NULL;
	HGLRC hRC = NULL;
}

COpenGL::~COpenGL()
{

}

BOOL COpenGL::SetupPixelFormat()
{
	int iPixelFormat;				
	PIXELFORMATDESCRIPTOR PFD;
	memset(&PFD, 0, sizeof(PIXELFORMATDESCRIPTOR));
	PFD.nSize = sizeof(PIXELFORMATDESCRIPTOR);
	PFD.nVersion = 1;
    PFD.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
    PFD.dwLayerMask = PFD_MAIN_PLANE;
	PFD.iPixelType = PFD_TYPE_RGBA;
	PFD.cColorBits = 32;
	PFD.cDepthBits = 32;
    PFD.cAccumBits = 0;
    PFD.cStencilBits = 0;
	if ((iPixelFormat=ChoosePixelFormat (hDC, &PFD))==FALSE)
	{
		MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
		return FALSE; 
	}
	if (SetPixelFormat(hDC, iPixelFormat, &PFD) == FALSE) 
{ 
		MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
		return FALSE; 
}
	return TRUE;
}

void COpenGL::InitGL(HWND hwnd)
{
	hDC=GetDC(hwnd);			
	SetupPixelFormat();		
	hRC=wglCreateContext(hDC);	
	wglMakeCurrent(hDC, hRC);
	SetupPixelFormat();
	glEnable(GL_TEXTURE_2D);
}					

void COpenGL::DeInitGL(HWND hwnd)
{
	wglMakeCurrent(NULL, NULL);
	if (hRC) wglDeleteContext(hRC);
	if (hDC) ReleaseDC(hwnd, hDC);
}

void COpenGL::InitViewPort(int w, int h)
{
	if (h==0)
	{
		h=1;						
	}
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)w/(GLfloat)h, 1.0f, 1500.0f);
	glMatrixMode(GL_MODELVIEW);	
	glLoadIdentity();	
}

void COpenGL::InitLight()
{
	glShadeModel(GL_SMOOTH);

	//  
	GLfloat light_position[]={1.0,1.0,1.0,0.0};
	GLfloat white_light[]={1.0,1.0,1.0,1.0};
	glLightfv(GL_LIGHT0,GL_POSITION,light_position);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,white_light);
	glLightfv(GL_LIGHT0,GL_SPECULAR,white_light);

	//...   
	GLfloat lmodel_ambient[]={1.0,1.0,1.0,1.0};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,lmodel_ambient);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	//  
	glEnable(GL_DEPTH_TEST);

	//     
//	GLfloat mat_specular_front[]={0.0,0.3,0.9,1.0};
//	GLfloat mat_ambient_front[]={0.0,0.1,0.3,1.0};
	GLfloat mat_specular_front[]={0.7f,0.7f,0.7f,1.0f};
	GLfloat mat_ambient_front[]={0.3f,0.3f,0.3f,1.0f};
	glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular_front);
	glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambient_front);

	//...   
//	GLfloat mat_specular_back[]={1.0,0.0,0.0,1.0};
//	GLfloat mat_ambient_back[]={0.5,0.1,0.1,1.0};
	GLfloat mat_specular_back[]={1.0,1.0,1.0,1.0};
	GLfloat mat_ambient_back[]={0.5,0.5,0.5,1.0};
	glMaterialfv(GL_BACK,GL_SPECULAR,mat_specular_back);
	glMaterialfv(GL_BACK,GL_AMBIENT,mat_ambient_back);
	
	//GL_SHININESS       
	GLfloat mat_shininess[]={50.0};
	glMaterialfv(GL_FRONT_AND_BACK,GL_SHININESS,mat_shininess);

	//   
//	GLdouble equation[]={0.4,0.5,-0.5,0.6};
//	glClipPlane(GL_CLIP_PLANE1, equation);
//	glEnable(GL_CLIP_PLANE1);

	//    
	glFrontFace(GL_CW);
  }


void COpenGL::SetVideomodeWnd(int HORZ_RES, int VERT_RES, int bpp)
{
	DEVMODE dmSettings;			
	memset(&dmSettings,0,sizeof(dmSettings));	
	if(!EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&dmSettings))
	{
		MessageBox(NULL, "Could Not Enum Display Settings", "Error", MB_OK);
		return;
	}
	dmSettings.dmPelsWidth	= HORZ_RES;			
	dmSettings.dmPelsHeight	= VERT_RES;				
	dmSettings.dmBitsPerPel	= bpp;					
	dmSettings.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT| DM_BITSPERPEL;
	int get_result = ChangeDisplaySettings(&dmSettings,CDS_FULLSCREEN);	
	if(get_result != DISP_CHANGE_SUCCESSFUL)
	{
		MessageBox(NULL, "Display Mode Not Compatible", "Error", MB_OK);
		PostQuitMessage(0);
	}
	ShowCursor(FALSE);	
}

void COpenGL::ResetVideoMode(void)
{
	ChangeDisplaySettings(NULL,0);
	ShowCursor(TRUE);
}



void COpenGL::UpDateScene(void)
{
	SwapBuffers(hDC);
}