#include "render.h"
#include "texture.h"
#include "LibTexture.h"
#include "myshader.h"
#include "frustum.h"
#include "InitGlExt.h"
#include "bitmap_fonts.h"

extern float g_fPS;
extern bool bbox_visible;
extern bool bshader_enable;
extern bool bFrustum;
extern bool b_vbo;
extern bool v_sync;
bool ppl=true;

mesh3ds mesh;


CTexture	*Ctex;
C_Clip frustum;
MyShader shader;

#define GL_TEXTURE0_ARB                     0x84C0
#define GL_TEXTURE1_ARB                     0x84C1


unsigned int * BufVertex;
unsigned int * BufVertexST;
unsigned int * BufTangent;
unsigned int * BufBinormal;
unsigned int * BufNormal;


CRender::CRender()
{
	draw_array = true;
}


void CRender::Init()
{

	b_vbo = IsExtensionSupported( "GL_ARB_vertex_buffer_object" );

	bshader_enable = IsExtensionSupported( "GL_ARB_shader_objects" );


	shader.InitShaders("shader.vert", "shader.frag");

	pScene = new T3DScene;
	pScene = mesh.Load3ds("data/flat2.3ds");

	Ctex = new CTexture[pScene->num_materials];
	char FileName[256];
	for (int i=0; i<pScene->num_materials; i++)
	{
		sprintf(FileName,"tex/%s",pScene->pMaterial[i].strFileName);

		Ctex[i].LoadTexture(FileName);

	}

	BufVertex = new unsigned int [pScene->num_objects];

	if (b_vbo==true)
	{
	for(i=0; i<pScene->num_objects; i++)
		{
			glGenBuffersARB( 1, &BufVertex[i] );
			glBindBufferARB(GL_ARRAY_BUFFER_ARB, BufVertex[i]);
			glBufferDataARB(GL_ARRAY_BUFFER_ARB, sizeof(tFace)*pScene->pObject[i].num_face, pScene->pObject[i].pFace, GL_STATIC_DRAW_ARB);	
		}
	}
}



void CRender::MainDraw()
{
	glClearColor(0.0, 0.0, 0.0, 1);					
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	if (v_sync==true) wglSwapIntervalEXT(1);
	else wglSwapIntervalEXT(0);



	for(int i=0;i<pScene->num_objects;i++)
	{

			if (bFrustum==true)
			{
				if (frustum.BBOXinVolume(pScene->pObject[i].min.x,
									 pScene->pObject[i].min.y,
									 pScene->pObject[i].min.z,
									 pScene->pObject[i].max.x,
									 pScene->pObject[i].max.y,
									 pScene->pObject[i].max.z) == true)
				{

					glColor4f(1,1,1,1);
					Ctex[pScene->pObject[i].TextureID].SetTexture();
					if (b_vbo==true)
					{
							if (bshader_enable==true) shader.Bind();
						glBindBufferARB(GL_ARRAY_BUFFER_ARB, BufVertex[i]);			
						glInterleavedArrays (GL_T2F_N3F_V3F, 0, (char *) NULL);
						glDrawArrays(GL_TRIANGLES,0, pScene->pObject[i].num_face*3);
							if (bshader_enable==true) shader.UnBind();
							if (bbox_visible==true) DrawBbox();
					}
					else
					{
							if (bshader_enable==true) shader.Bind();
						glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);			
						glInterleavedArrays (GL_T2F_N3F_V3F, 0, pScene->pObject[i].pFace);
						glDrawArrays(GL_TRIANGLES,0, pScene->pObject[i].num_face*3);
							if (bshader_enable==true) shader.UnBind();
							if (bbox_visible==true) DrawBbox();
					}
			}
		}

	}

			PrintInfo();
}



CRender::~CRender()
{


	glDeleteBuffersARB( pScene->num_objects, BufVertex );	
	
	for (int i=0; i<pScene->num_materials; i++)
	{
		Ctex[i].DeleteTexture();
	}
}


bool CRender::IsExtensionSupported( char* szTargetExtension )
{
	const unsigned char *pszExtensions = NULL;
	const unsigned char *pszStart;
	unsigned char *pszWhere, *pszTerminator;


	pszWhere = (unsigned char *) strchr( szTargetExtension, ' ' );
	if( pszWhere || *szTargetExtension == '\0' )
		return false;

	// Get Extensions String
	pszExtensions = glGetString( GL_EXTENSIONS );

	// Search The Extensions String For An Exact Copy
	pszStart = pszExtensions;
	for(;;)
	{
		pszWhere = (unsigned char *) strstr( (const char *) pszStart, szTargetExtension );
		if( !pszWhere )
			break;
		pszTerminator = pszWhere + strlen( szTargetExtension );
		if( pszWhere == pszStart || *( pszWhere - 1 ) == ' ' )
			if( *pszTerminator == ' ' || *pszTerminator == '\0' )
				return true;
		pszStart = pszTerminator;
	}
	return false;
}


void CRender::PrintInfo()
{
	bshader_enable;
    static char infoString[] = "Press F1 to toggle culling";
	static char infoString2[] = "Press F2 to toggle lighting by Pixel/Vertex Shader";
	static char infoString3[] = "Press F3 to toggle drawing Bounding Box";
	static char infoString4[] = "Press F4 to toggle VBO (Vertex Buffer Object)";
	static char infoString5[] = "Press F5 to toggle VsynC";
	static char cull_mode[16];
	static char shader_mode[40];
	static char fr[40];
	static char b_box[40];
	static char vbo[40];
	static char vsync[40];

	sprintf( cull_mode, "Culling mode: %s",  "OFF" );
	if (bFrustum) sprintf( cull_mode, "Culling mode: %s",  "ON" );

	sprintf( shader_mode, "Pixel/Vertex Shader: %s",  "disabled" );
	if (bshader_enable) sprintf( shader_mode, "Pixel/Vertex Shader: %s",  "enabled" );

	sprintf( fr, "Current framerate: %i FPS",  int(g_fPS) );

	sprintf( b_box, "Bounding Box: %s",  "invisible" );
	if (bbox_visible) sprintf( b_box, "Bounding Box: %s",  "visible" );

	sprintf( vbo, "VBO: %s",  "disabled" );
	if (b_vbo) sprintf( vbo, "VBO: %s",  "enabled" );

	sprintf( vsync, "VSYNC: %s",  "OFF" );
	if (v_sync) sprintf( vsync, "VSYNC: %s",  "ON" );

    beginRenderText( 640, 480 );
    {
        glColor3f( 1.0f, 1.0f, 1.0f );
        renderText( 0, 10, BITMAP_FONT_TYPE_HELVETICA_12, infoString );
		renderText( 500, 10, BITMAP_FONT_TYPE_HELVETICA_12, cull_mode );

		glColor3f( 1.0f, 1.0f, 1.0f );
        renderText( 0, 22, BITMAP_FONT_TYPE_HELVETICA_12, infoString2 );
		renderText( 500, 22, BITMAP_FONT_TYPE_HELVETICA_12, shader_mode );

		glColor3f( 1.0f, 1.0f, 1.0f );
		renderText( 0, 34, BITMAP_FONT_TYPE_HELVETICA_12, infoString3 );
		renderText( 500, 34, BITMAP_FONT_TYPE_HELVETICA_12, b_box );

		glColor3f( 1.0f, 1.0f, 1.0f );
        renderText( 0, 478, BITMAP_FONT_TYPE_HELVETICA_12, fr );

		glColor3f(1.0f, 1.0f, 1.0f );
		renderText( 0, 46, BITMAP_FONT_TYPE_HELVETICA_12, infoString4 );
		renderText( 500, 46, BITMAP_FONT_TYPE_HELVETICA_12, vbo );

		glColor3f(1.0f, 1.0f, 1.0f );
		renderText( 0, 58, BITMAP_FONT_TYPE_HELVETICA_12, infoString5 );
		renderText( 500, 58, BITMAP_FONT_TYPE_HELVETICA_12, vsync );

   	}
    endRenderText();
}



void CRender::DrawBbox()
{
	glBegin(GL_LINES);
	for (int i=0; i<pScene->num_objects; i++)
	{
		glColor4f(0,1,0,0);
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].min.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].min.y, pScene->pObject[i].min.z);

	
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].min.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].max.y, pScene->pObject[i].min.z);

		
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].min.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].min.y, pScene->pObject[i].max.z);
//*****************************
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].max.y, pScene->pObject[i].max.z);
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].max.y, pScene->pObject[i].max.z);

	
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].max.y, pScene->pObject[i].max.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].min.y, pScene->pObject[i].max.z);

		
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].max.y, pScene->pObject[i].max.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].max.y, pScene->pObject[i].min.z);

//********************************
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].min.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].min.y, pScene->pObject[i].max.z);

		
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].max.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].max.y, pScene->pObject[i].max.z);


		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].max.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].max.y, pScene->pObject[i].min.z);


		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].min.y, pScene->pObject[i].max.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].min.y, pScene->pObject[i].max.z);

		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].max.y, pScene->pObject[i].max.z);
		glVertex3f(pScene->pObject[i].min.x, pScene->pObject[i].min.y, pScene->pObject[i].max.z);

		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].max.y, pScene->pObject[i].min.z);
		glVertex3f(pScene->pObject[i].max.x, pScene->pObject[i].min.y, pScene->pObject[i].min.z);
	}
	glEnd();
}



void CRender::Free()
{
	delete pScene;
}