#include "texture.h"
#pragma comment (lib, "jpeg.lib")

#define TGA_RGB			2
#define TGA_A			3
#define TGA_RLE			10
extern HWND hwnd;


struct  TgaHeader
{
    unsigned char	idLength;
    unsigned char	colormapType;
    unsigned char	imageType;
    unsigned char	colormapIndex;
	unsigned char	colormapIndex1;
    unsigned char	colormapLength;
	unsigned char	colormapLength1;

    unsigned char	colormapEntrySize;
    unsigned short  xOrigin;
    unsigned short  yOrigin;
    unsigned short  width;
    unsigned short  height;
    unsigned char	pixelSize;
    unsigned char	imageDesc;
};



CTexture::CTexture()
{
	texID=-1;				
	ImageDATA=NULL;
	pImage = NULL;
	pFile = NULL;
}


CTexture :: CTexture ( int theWidth, int theHeight, int theNumComponents )
{
    width         = theWidth;
    height        = theHeight;
    numComponents = theNumComponents;
    ImageDATA          = (byte *) malloc ( width * height * numComponents );
    levels        = 1;
    compressed    = false;
    mipmapped     = false;

    switch ( numComponents )
    {
        case 1:
            format = GL_ALPHA;
            break;

        case 3:
            format = GL_RGB;
            break;

        case 4:
            format = GL_RGBA;
            break;

        default:
            format = -1;
    }
}



CTexture::~CTexture()
{

}


bool CTexture::LoadTexture(LPSTR strFileName)

{
	if(strFileName==NULL) return false;

	CTexture * tex = NULL;

	tex= new CTexture;

    const char * ext = strrchr ( strFileName, '.' );

	if ( ext == NULL ) return false;


    if ( !stricmp ( ext + 1, "tga" ) )
    {
		if (!LoadTGA(strFileName)) return false;
		BuildTexture(true);
		if (ImageDATA) free(ImageDATA);
    }


    if ( !stricmp ( ext + 1, "jpg" ) )
    {
		if (!LoadJPG(strFileName)) return false;
		BuildTexture(true);
		if (pImage)
		{
			delete pImage;
		}
		if (ImageDATA) free(ImageDATA);
	}

   if ( !stricmp ( ext + 1, "bmp" ) )
    {
		if (!Read_BMP_RGB(strFileName)) return false;
		BuildTexture(true);
		if (ImageDATA) free(ImageDATA);
    }
	return true;
}



bool CTexture::Read_BMP_RGB(char* filename)
{		
	unsigned char bmpHEADER[54];
	unsigned int  ImageSIZE;	
	unsigned int  temp;			

	if((pFile = fopen(filename, "rb")) == NULL)	
	{
		MessageBox(hwnd, "Unable to load BMP_RGB File!", "Error", MB_OK);
		return false;
	}

	if((fread(bmpHEADER,1,sizeof(bmpHEADER),pFile))== NULL)
	{
		MessageBox(hwnd, "Unable to read BMP File Header!", "Error", MB_OK);	
		fclose(pFile);
		return false;
	}
	width  = bmpHEADER[19]*256+bmpHEADER[18];
	height = bmpHEADER[23]*256+bmpHEADER[22];

	bpp=3;							



	ImageSIZE = width * height * bpp;
	ImageDATA = (GLubyte *)malloc(ImageSIZE);	
	if(ImageDATA==NULL || fread(ImageDATA, 1, ImageSIZE, pFile)!=ImageSIZE)
	{
		if(ImageDATA!=NULL)	free(ImageDATA);
		fclose(pFile);
		return false;
	}
	for(GLuint i=0; i<int(ImageSIZE); i+=bpp)

	{	
		temp=ImageDATA[i];
		ImageDATA[i] = ImageDATA[i + 2];
		ImageDATA[i + 2] = temp;
	}

	texTYPE=GL_RGB;
	numComponents = 3;

	if (pFile) fclose (pFile);
	return true;
} 




bool CTexture::LoadTGA(char *filename)
{
	unsigned char fileSIGN[12];			
	unsigned char tgaHEADER[6];
	unsigned int  ImageSIZE;	
	unsigned int  temp;			

	if((pFile = fopen(filename, "rb")) == NULL)	
	{
		MessageBox(hwnd, "Unable to load TGA File!", "Error", MB_OK);
		return false;
	}

	if((fread(fileSIGN,1,sizeof(fileSIGN), pFile))== NULL)
	{
		MessageBox(hwnd, "Unable to read TGA File Signature!", "Error", MB_OK);	
		fclose(pFile);
		return false;
	}
	if(fileSIGN[2]==TGA_RLE)			{
		MessageBox(hwnd, "This is TGA RLE Compressed File! Loading not possible", "Error", MB_OK);
		fclose(pFile);
		return false;
	}
	if((fread(tgaHEADER,1,sizeof(tgaHEADER),pFile))== NULL)
	{
		MessageBox(hwnd, "Unable to read TGA File Header!", "Error", MB_OK);	
		fclose(pFile);
		return false;
	}
	width=tgaHEADER[1]*256+tgaHEADER[0];
	height=tgaHEADER[3]*256+tgaHEADER[2];
	bpp=tgaHEADER[4]/8;							
	if (bpp==3) 
	{
		texTYPE=GL_RGB;
		numComponents = 3;
	}
	if (bpp==4) 
	{
		texTYPE=GL_RGBA;
		numComponents = 4;
	}
	if(width<=0 || height <=0 ||					
	  (tgaHEADER[4]!=24 && tgaHEADER[4]!=32))	
	{
		MessageBox(hwnd, "Wrong size or depth of TGA File!", "Error", MB_OK);
		fclose(pFile);
		return false;
	}
	ImageSIZE = width * height * bpp;
	ImageDATA = (GLubyte *)malloc(ImageSIZE);	
	if(ImageDATA==NULL || fread(ImageDATA, 1, ImageSIZE, pFile)!=ImageSIZE)
	{
		if(ImageDATA!=NULL)	free(ImageDATA);
		fclose(pFile);
		return false;
	}
	for(GLuint i=0; i<int(ImageSIZE); i+=bpp)

	{	
		temp=ImageDATA[i];
		ImageDATA[i] = ImageDATA[i + 2];
		ImageDATA[i + 2] = temp;
	}
	if (pFile) fclose (pFile);
	return true;
}


void CTexture::DecodeJPG(jpeg_decompress_struct* cinfo)
{
	jpeg_read_header(cinfo, TRUE);
	

	jpeg_start_decompress(cinfo);


	pImage->rowSpan = cinfo->image_width * cinfo->num_components;
	pImage->sizeX   = cinfo->image_width;
	pImage->sizeY   = cinfo->image_height;
	

	ImageDATA = new unsigned char[pImage->rowSpan * pImage->sizeY];
		

	unsigned char** rowPtr = new unsigned char*[pImage->sizeY];
	for (int i = 0; i < pImage->sizeY; i++)
		rowPtr[i] = &(ImageDATA[i*pImage->rowSpan]);


	int rowsRead = 0;
	while (cinfo->output_scanline < cinfo->output_height) 
	{

		rowsRead += jpeg_read_scanlines(cinfo, &rowPtr[rowsRead], cinfo->output_height - rowsRead);
	}
	

	delete [] rowPtr;

	jpeg_finish_decompress(cinfo);
}


bool  CTexture::LoadJPG(char *filename)
{
	struct jpeg_decompress_struct cinfo;


	if((pFile = fopen(filename, "rb")) == NULL) 
	{

		MessageBox(hwnd, "Unable to load JPG File!", "Error", MB_OK);
		return NULL;
	}
	

	jpeg_error_mgr jerr;


	cinfo.err = jpeg_std_error(&jerr);
	

	jpeg_create_decompress(&cinfo);
	
	
	jpeg_stdio_src(&cinfo, pFile);
	

	pImage = new tImageJPG;


	DecodeJPG(&cinfo);
	

	jpeg_destroy_decompress(&cinfo);
	

	fclose(pFile);
	if(pImage == NULL)	return false;
	width =  pImage->sizeX;
	height = pImage->sizeY;
	texTYPE = GL_RGB;
	numComponents = 3;
	return true;
}


void CTexture::BuildTexture(bool wrap)
{
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap?GL_REPEAT:GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap?GL_REPEAT:GL_CLAMP);
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE, GL_MODULATE);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, texTYPE, GL_UNSIGNED_BYTE, ImageDATA);
}



void CTexture::SetTexture()
{
	glBindTexture(GL_TEXTURE_2D, texID);
}

void CTexture::DeleteTexture()
{
	glDeleteTextures(1, &texID);
}







bool    CTexture::saveScreenShot ( const char * fileName )
{
    unsigned short    width  = 640;
    unsigned short    height = 480;
	unsigned short		bpp	=3;
    ImageDATA    = (byte *) malloc ( 3 * width * height );

    if ( ImageDATA == NULL )  return false;

    glReadBuffer ( GL_FRONT );
    glReadPixels ( 0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE, ImageDATA );

 	unsigned char fileSIGN[12];			
	unsigned char tgaHEADER[6];

    int file = open ( fileName, O_WRONLY | O_BINARY | O_CREAT, S_IWRITE );

    if ( file == -1 )
    {
        free ( ImageDATA );

        return false;
    }
 
	memset ( fileSIGN, '\0', sizeof ( fileSIGN ) );
	fileSIGN[2]=2;
	fileSIGN[7]=0x18;
    write ( file, fileSIGN, sizeof ( fileSIGN ) );

	memset ( tgaHEADER, '\0', sizeof ( tgaHEADER ) );
	tgaHEADER[1]= width/256;
	tgaHEADER[0]=width-tgaHEADER[1]*256;
	tgaHEADER[3]= height/256;
	tgaHEADER[2]=height-tgaHEADER[3]*256;
	tgaHEADER[4]=24;
	tgaHEADER[5]=0x20;
	write ( file, tgaHEADER, sizeof ( tgaHEADER ) );
	
    int     ImageSIZE = width * height*bpp;
    byte  * pixel     = ImageDATA;
    byte    temp;

    for ( int i = 0; i < ImageSIZE; i+=3 )
    {
 		temp=ImageDATA[i];
		ImageDATA[i] = ImageDATA[i + 2];
		ImageDATA[i + 2] = temp;	
    }

	GLubyte *ImageDATA2;	
	ImageDATA2    = (byte *) malloc ( 3 * width * height );

    for (i = 0; i < ImageSIZE; i+=3 )
    {
 		ImageDATA2[i]=ImageDATA[ImageSIZE-i-3];
		ImageDATA2[i+1]=ImageDATA[ImageSIZE-i-2];
		ImageDATA2[i+2]=ImageDATA[ImageSIZE-i-1];
    }


    write ( file, ImageDATA2, width * height * 3 );
    close ( file );

	free(ImageDATA2);
	
    return true;
}


int CTexture::SaveTGA(char *name,int width,int height)
 {
    int i,j;
    unsigned char *buffer;
    FILE    *FileTGA = fopen(name,"wb");
    if(!FileTGA) return 0;
    buffer = (unsigned char*)malloc(18 + width * height * 3);
    memset(buffer,0,18);
    buffer[2] = 2;
    buffer[12] = width & 0xFF;
    buffer[13] = width >> 8;
    buffer[14] = height & 0xFF;
    buffer[15] = height >> 8;
    buffer[16] = 24;
    buffer[17] = 0x20;


	GLubyte * data    = (byte *) malloc ( 3 * width * height );

    glReadBuffer ( GL_FRONT );
    glReadPixels ( 0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE, data );

    memcpy(buffer + 18,data,width * height * 3);
    for(i = 18; i < 18 + width * height * 3 ; i += 3) {
        j = buffer[i];
        buffer[i] = buffer[i + 2];
        buffer[i + 2] = j;
    }
    fwrite(buffer,1,18 + width * height * 3,FileTGA);

  
	fclose(FileTGA);
	free(data);
    free(buffer);
    return 1;
} 