#ifndef _vector_h_	//      ,      
#define _vector_h_

#include <math.h>



class Vector2D
{
public:
    float x, y;			//   -   CVector

public:

	Vector2D(float X, float Y): x(X), y(Y){}


	Vector2D(): x(0), y(0){}		


    Vector2D operator + (const Vector2D &V) const
    {
        return Vector2D(x + V.x, y + V.y);
    }

   Vector2D operator += (const Vector2D &V)
    {
        x += V.x; y += V.y;
        return *this;
    }

	Vector2D operator -= (const Vector2D &V)
    {
        x -= V.x; y -= V.y;
        return *this;
    }


	Vector2D operator - () const
    {
	  return Vector2D(-x, -y);
    }


	Vector2D operator - (const Vector2D &V) const
    {
	  return Vector2D(x - V.x, y - V.y);
    }


	Vector2D &operator * (const Vector2D &V)
    {
        x *= V.x; y *= V.y;
        return *this;
    }


	Vector2D operator * (float f) const 
    {
        return Vector2D(x * f, y * f); 
    }


    float operator | (const Vector2D &V) const
    {
        return x*V.x + y*V.y;
    }
public:
    float Magnitude()
    {
        return (float)sqrt(x*x + y*y);
    }


	Vector2D &Normalize()
    {
		float veclen = Magnitude();
        x /= veclen;
        y /= veclen;
        return *this;
    }


    Vector2D GetNormalized()
    {
        return Vector2D(*this).Normalize();
    }
};



class Vector3D
{
public:
    float x, y, z;			//   -   CVector

public:

	Vector3D(float X, float Y, float Z): x(X), y(Y), z(Z) {}


	Vector3D(): x(0), y(0), z(0) {}		


    Vector3D operator + (const Vector3D &V) const
    {
        return Vector3D(x + V.x, y + V.y, z + V.z);
    }

   Vector3D operator += (const Vector3D &V)
    {
        x += V.x; y += V.y; z += V.z;
        return *this;
    }

	Vector3D operator -= (const Vector3D &V)
    {
        x -= V.x; y -= V.y; z -= V.z;
        return *this;
    }


	Vector3D operator - () const
    {
	  return Vector3D(-x, -y, -z);
    }


	Vector3D operator - (const Vector3D &V) const
    {
	  return Vector3D(x - V.x, y - V.y, z - V.z);
    }


	Vector3D &operator * (const Vector3D &V)
    {
        x *= V.x; y *= V.y; z *= V.z;
        return *this;
    }


	Vector3D operator * (float f) const 
    {
        return Vector3D(x * f, y * f, z * f); 
    }


    Vector3D operator^(const Vector3D& V) const
    {
        return Vector3D(y*V.z - z*V.y, z*V.x - x*V.z, x*V.y - y*V.x);
    }

    float operator | (const Vector3D &V) const
    {
        return x*V.x + y*V.y + z*V.z;
    }
public:
    float Magnitude()
    {
        return (float)sqrt(x*x + y*y + z*z);
    }


    Vector3D &Normalize()
    {
		float veclen = Magnitude();
        x /= veclen;
        y /= veclen;
        z /= veclen;
        return *this;
    }


    Vector3D GetNormalized()
    {
        return Vector3D(*this).Normalize();
    }
};


#endif _vector_h_
